@@ -79,6 +79,14 @@ setMethod(
     }
 )
 
+setMethod(
+    f = "print",
+    signature = signature(x = 'lca'),
+    definition = function(x, ...){
+        cat('Item object of class:', class(x))
+    }
+)
+
 setMethod(
     f = "print",
     signature = signature(x = 'GroupPars'),
@@ -167,6 +175,14 @@ setMethod(
     }
 )
 
+setMethod(
+    f = "show",
+    signature = signature(object = 'lca'),
+    definition = function(object){
+        print(object)
+    }
+)
+
 setMethod(
     f = "show",
     signature = signature(object = 'GroupPars'),
@@ -258,6 +274,14 @@ setMethod(
     }
 )
 
+setMethod(
+    f = "ExtractLambdas",
+    signature = signature(x = 'lca'),
+    definition = function(x){
+        x@par[1L:x@nfact]
+    }
+)
+
 #----------------------------------------------------------------------------
 setMethod(
     f = "ExtractZetas",
@@ -455,6 +479,16 @@ setMethod(
     }
 )
 
+setMethod(
+    f = "GenRandomPars",
+    signature = signature(x = 'lca'),
+    definition = function(x){
+        par <- rnorm(length(x@par))
+        x@par[x@est] <- par[x@est]
+        x
+    }
+)
+
 setMethod(
     f = "GenRandomPars",
     signature = signature(x = 'GroupPars'),
@@ -619,6 +653,19 @@ P.ideal <- function(par, Theta, ot = 0)
     return(P)
 }
 
+setMethod(
+    f = "ProbTrace",
+    signature = signature(x = 'lca', Theta = 'matrix'),
+    definition = function(x, Theta, useDesign = TRUE, ot=0){
+        P <- P.lca(x@par, Theta=Theta, score=x@score)
+        return(P)
+    }
+)
+
+P.lca <- function(par, Theta, score, returnNum = FALSE){
+    return(.Call('lcaTraceLinePts', par, Theta, score, returnNum))
+}
+
 #----------------------------------------------------------------------------
 ## initialize for custom items
 setMethod("initialize",
