@@ -1,194 +1,203 @@
 shinyItemplot <- function(){
-    require(latticeExtra)
-    
+
     ret <- list(
-        
-        ui = pageWithSidebar(
-            
+
+        ui = shiny::pageWithSidebar(
+
             # Application title
-            headerPanel("Item plots in mirt"),
-            
-            sidebarPanel(
-                
-                h5('Select an internal mirt item class, the type of plot to display, whether the plot
+            shiny::headerPanel("Item plots in mirt"),
+
+            shiny::sidebarPanel(
+
+                shiny::h5('Select an internal mirt item class, the type of plot to display, whether the plot
                    inputs should be the classical or modern IRT parameterizations, and whether the plot
                    should be unidimensional or multidimensional. See ?mirt for more details.'),
-                
-                selectInput(inputId = "itemclass",
+
+                shiny::selectInput(inputId = "itemclass",
                             label = "Class of mirt item:",
-                            choices = c('dich' = 'dich', 
-                                        'graded' = 'graded', 
-                                        'nominal' = 'nominal', 
-                                        'gpcm' = 'gpcm', 
-                                        'partcomp' = 'partcomp'),
+                            choices = c('dich' = 'dich',
+                                        'graded' = 'graded',
+                                        'nominal' = 'nominal',
+                                        'gpcm' = 'gpcm',
+                                        'partcomp' = 'partcomp',
+                                        'ideal' = 'ideal'),
                             selected = 'dich'),
-                
-                selectInput(inputId = "plottype",
+
+                shiny::selectInput(inputId = "plottype",
                             label = "Type of plot to display:",
                             choices = c('trace', 'info', 'score', 'infocontour', 'SE', 'infoSE', 'tracecontour'),
                             selected = 'trace'),
-                
-                numericInput("theta_lim_low", "Theta lower range:", -6,
+
+                shiny::numericInput("theta_lim_low", "Theta lower range:", -6,
                              min = -35, max = 35),
-                
-                numericInput("theta_lim_high", "Theta upper range:", 6,
+
+                shiny::numericInput("theta_lim_high", "Theta upper range:", 6,
                              min = -35, max = 35),
-                
-                checkboxInput(inputId = "classical",
+
+                shiny::checkboxInput(inputId = "classical",
                               label = "Use traditional IRT parameterization inputs? \n
                               Only applicable to class dich, graded, gpcm, and nominal.",
                               value = FALSE),
-                
-                checkboxInput(inputId = "nfact",
+
+                shiny::checkboxInput(inputId = "nfact",
                               label = "Multidimensional?",
                               value = FALSE),
-                
+
                 #-------------------------------------------------------------------------
-                
-                conditionalPanel(condition = "input.classical == false",
-                                 
-                                 sliderInput(inputId = "a1par",
+
+                shiny::conditionalPanel(condition = "input.classical == false",
+
+                                        shiny::sliderInput(inputId = "a1par",
                                              label = "a1 value:",
                                              min = -3, max = 3, value = 1, step = 0.2),
-                                 
-                                 conditionalPanel(condition = "input.nfact == true",
-                                                  sliderInput(inputId = "a2par",
+
+                                        shiny::conditionalPanel(condition = "input.nfact == true",
+                                                                shiny::sliderInput(inputId = "a2par",
                                                               label = "a2 value:",
                                                               min = -3, max = 3, value = 1, step = 0.2)),
-                                 
-                                 conditionalPanel(condition = "input.itemclass == 'dich'",
-                                                  sliderInput(inputId = "dpar",
+
+                                        shiny::conditionalPanel(condition = "input.itemclass == 'dich'",
+                                                                shiny::sliderInput(inputId = "dpar",
                                                               label = "d value:",
                                                               min = -5, max = 5, value = 0, step = 0.25),
-                                                  sliderInput(inputId = "gpar",
+                                                              shiny::sliderInput(inputId = "gpar",
                                                               label = "g value:",
                                                               min = 0, max = 1, value = 0, step = 0.05),
-                                                  sliderInput(inputId = "upar",
+                                                              shiny::sliderInput(inputId = "upar",
                                                               label = "u value:",
-                                                              min = 0, max = 1, value = 1, step = 0.05)                                         
+                                                              min = 0, max = 1, value = 1, step = 0.05)
                                  ),
-                                 
-                                 conditionalPanel(condition = "input.itemclass != 'dich'",
-                                                  conditionalPanel(condition = "input.itemclass == 'gpcm' || 
+
+                                 shiny::conditionalPanel(condition = "input.itemclass == 'ideal'",
+                                                         shiny::sliderInput(inputId = "idpar",
+                                                              label = "d value:",
+                                                              min = -5, max = 5, value = 0, step = 0.25)
+                                 ),
+
+                                 shiny::conditionalPanel(condition = "input.itemclass != 'dich'",
+                                                         shiny::conditionalPanel(condition = "input.itemclass == 'gpcm' ||
                                                                    input.itemclass == 'nominal'",
-                                                                   sliderInput(inputId = "d0par",
+                                                                                 shiny::sliderInput(inputId = "d0par",
                                                                                label = "d0 value (default fixed at 0):",
                                                                                min = -5, max = 5, value = 0, step = 0.25)
                                                   ),
-                                                  sliderInput(inputId = "d1par",
-                                                              label = "d1 value:",
-                                                              min = -5, max = 5, value = 1, step = 0.25),
-                                                  sliderInput(inputId = "d2par",
-                                                              label = "d2 value:",
-                                                              min = -5, max = 5, value = 0, step = 0.25),
-                                                  conditionalPanel(condition = "input.itemclass != 'partcomp'",
-                                                                   sliderInput(inputId = "d3par",
-                                                                               label = "d3 value:",
-                                                                               min = -5, max = 5, value = -1, step = 0.25)
+
+                                                  shiny::conditionalPanel(condition = "input.itemclass != 'ideal'",
+                                                                          shiny::sliderInput(inputId = "d1par",
+                                                                  label = "d1 value:",
+                                                                  min = -5, max = 5, value = 1, step = 0.25),
+                                                                  shiny::sliderInput(inputId = "d2par",
+                                                                  label = "d2 value:",
+                                                                  min = -5, max = 5, value = 0, step = 0.25),
+                                                                  shiny::conditionalPanel(condition = "input.itemclass != 'partcomp'",
+                                                                                          shiny::sliderInput(inputId = "d3par",
+                                                                                   label = "d3 value:",
+                                                                                   min = -5, max = 5, value = -1, step = 0.25)
+                                                      )
                                                   )
                 ),
-                                 
-                                 conditionalPanel(condition = "input.itemclass == 'nominal'",                         
-                                                  sliderInput(inputId = "ak0par",
+
+                shiny::conditionalPanel(condition = "input.itemclass == 'nominal'",
+                                        shiny::sliderInput(inputId = "ak0par",
                                                               label = "ak0 value (default fixed at 0):",
                                                               min = -3, max = 3, value = 0, step = 0.2),
-                                                  sliderInput(inputId = "ak1par",
+                                        shiny::sliderInput(inputId = "ak1par",
                                                               label = "ak0 value:",
                                                               min = -3, max = 3, value = 1, step = 0.2),
-                                                  sliderInput(inputId = "ak2par",
+                                        shiny::sliderInput(inputId = "ak2par",
                                                               label = "ak2 value:",
                                                               min = -3, max = 3, value = 2, step = 0.2),
-                                                  sliderInput(inputId = "ak3par",
+                                        shiny::sliderInput(inputId = "ak3par",
                                                               label = "ak3 value (default fixed at (ncat-1)):",
                                                               min = -3, max = 3, value = 3, step = 0.2)
                                  )
                 ),
-                
-                conditionalPanel(condition = "input.classical == true",       
-                                 
-                                 conditionalPanel(condition = "input.itemclass == 'dich'",
-                                                  sliderInput(inputId = "a1parc",
+
+                shiny::conditionalPanel(condition = "input.classical == true",
+
+                                        shiny::conditionalPanel(condition = "input.itemclass == 'dich'",
+                                                                shiny::sliderInput(inputId = "a1parc",
                                                               label = "a value:",
                                                               min = -3, max = 3, value = 1, step = 0.2),
-                                                  sliderInput(inputId = "bpar", 
+                                                              shiny::sliderInput(inputId = "bpar",
                                                               label = "b value:",
                                                               min = -5, max = 5, value = 0, step = 0.25),
-                                                  sliderInput(inputId = "gparc",
+                                                              shiny::sliderInput(inputId = "gparc",
                                                               label = "g value:",
                                                               min = 0, max = 1, value = 0, step = 0.05),
-                                                  sliderInput(inputId = "uparc",
+                                                              shiny::sliderInput(inputId = "uparc",
                                                               label = "u value:",
-                                                              min = 0, max = 1, value = 1, step = 0.05)                                         
-                                 ),                   
-                                 
-                                 conditionalPanel(condition = "input.itemclass == 'graded'",
-                                                  sliderInput(inputId = "a1pard",
+                                                              min = 0, max = 1, value = 1, step = 0.05)
+                                 ),
+
+                                 shiny::conditionalPanel(condition = "input.itemclass == 'graded'",
+                                                         shiny::sliderInput(inputId = "a1pard",
                                                               label = "a value:",
                                                               min = -3, max = 3, value = 1, step = 0.2),
-                                                  sliderInput(inputId = "bpar1d", 
+                                                         shiny::sliderInput(inputId = "bpar1d",
                                                               label = "b1 value:",
                                                               min = -5, max = 5, value = -1, step = 0.25),
-                                                  sliderInput(inputId = "bpar2d", 
+                                                         shiny::sliderInput(inputId = "bpar2d",
                                                               label = "b2 value:",
                                                               min = -5, max = 5, value = 0, step = 0.25),
-                                                  sliderInput(inputId = "bpar3d", 
+                                                         shiny::sliderInput(inputId = "bpar3d",
                                                               label = "b3 value:",
                                                               min = -5, max = 5, value = 1, step = 0.25)
                                  ),
-                                 
-                                 conditionalPanel(condition = "input.itemclass == 'gpcm'",
-                                                  sliderInput(inputId = "a1pare",
+
+                                 shiny::conditionalPanel(condition = "input.itemclass == 'gpcm'",
+                                                         shiny::sliderInput(inputId = "a1pare",
                                                               label = "a value:",
                                                               min = -3, max = 3, value = 1, step = 0.2),
-                                                  sliderInput(inputId = "bpar1e", 
+                                                         shiny::sliderInput(inputId = "bpar1e",
                                                               label = "b1 value:",
                                                               min = -5, max = 5, value = -1, step = 0.25),
-                                                  sliderInput(inputId = "bpar2e", 
+                                                         shiny::sliderInput(inputId = "bpar2e",
                                                               label = "b2 value:",
                                                               min = -5, max = 5, value = 0, step = 0.25),
-                                                  sliderInput(inputId = "bpar3e", 
+                                                         shiny::sliderInput(inputId = "bpar3e",
                                                               label = "b3 value:",
                                                               min = -5, max = 5, value = 1, step = 0.25)
                                  ),
-                                 
-                                 conditionalPanel(condition = "input.itemclass == 'nominal'",
-                                                  sliderInput(inputId = "a1parf",
+
+                                 shiny::conditionalPanel(condition = "input.itemclass == 'nominal'",
+                                                         shiny::sliderInput(inputId = "a1parf",
                                                               label = "a1 value:",
                                                               min = -3, max = 3, value = -1.4, step = 0.2),
-                                                  sliderInput(inputId = "a2parf", 
+                                                         shiny::sliderInput(inputId = "a2parf",
                                                               label = "a2 value:",
                                                               min = -3, max = 3, value = -0.4, step = 0.2),
-                                                  sliderInput(inputId = "a3parf",
+                                                         shiny::sliderInput(inputId = "a3parf",
                                                               label = "a3 value:",
                                                               min = -3, max = 3, value = 0.4, step = 0.2),
-                                                  sliderInput(inputId = "a4parf",
+                                                         shiny::sliderInput(inputId = "a4parf",
                                                               label = "a4 value:",
                                                               min = -3, max = 3, value = 1.4, step = 0.2),
-                                                  sliderInput(inputId = "bpar1f", 
+                                                         shiny::sliderInput(inputId = "bpar1f",
                                                               label = "b1 value:",
                                                               min = -5, max = 5, value = -1, step = 0.25),
-                                                  sliderInput(inputId = "bpar2f", 
+                                                         shiny::sliderInput(inputId = "bpar2f",
                                                               label = "b2 value:",
                                                               min = -5, max = 5, value = 0, step = 0.25),
-                                                  sliderInput(inputId = "bpar3f", 
+                                                         shiny::sliderInput(inputId = "bpar3f",
                                                               label = "b3 value:",
                                                               min = -5, max = 5, value = 1, step = 0.25),
-                                                  sliderInput(inputId = "bpar4f", 
+                                                         shiny::sliderInput(inputId = "bpar4f",
                                                               label = "b4 value:",
                                                               min = -5, max = 5, value = 1.5, step = 0.25)
-                                 )                                     
+                                 )
                 )
                 ),
-            
-            mainPanel(
-                verbatimTextOutput("coefs"),
-                plotOutput(outputId = "main_plot", height = "700px", width = "700px")
+
+            shiny::mainPanel(
+                shiny::verbatimTextOutput("coefs"),
+                shiny::plotOutput(outputId = "main_plot", height = "700px", width = "700px")
             )
-            
+
                 ),
-        
+
         server = function(input, output) {
-            
+
             genmod <- function(input){
                 set.seed(1234)
                 itemclass <- c(input$itemclass, input$itemclass)
@@ -199,7 +208,8 @@ shinyItemplot <- function(){
                                    nominal='nominal',
                                    nestlogit='2PLNRM',
                                    partcomp='PC2PL',
-                                   nestlogit='2PLNRM')
+                                   nestlogit='2PLNRM',
+                                   ideal='ideal')
                 nominal <- NULL
                 model <- 1
                 if(input$nfact) model <- 2
@@ -230,7 +240,7 @@ shinyItemplot <- function(){
                 }
                 dat <- simdata(a=a, d=d, N=100,
                                itemtype=itemclass, nominal=nominal)
-                sv <- mirt(dat, model, itemtype=itemtype, pars = 'values', key=c(1, NA), 
+                sv <- mirt(dat, model, itemtype=itemtype, pars = 'values', key=c(1, NA),
                            technical=list(message=FALSE))
                 sv$est <- FALSE
                 mod <- mirt(dat, model, itemtype=itemtype, pars=sv, key=c(1, NA),
@@ -244,7 +254,7 @@ shinyItemplot <- function(){
                     } else if(itemclass[1L] == 'gpcm'){
                         par <- c(input$a1pare, input$bpar1e, input$bpar2e, input$bpar3e)
                     } else if(itemclass[1L] == 'nominal'){
-                        par <- c(input$a1parf, input$a2parf, input$a3parf, input$a4parf, 
+                        par <- c(input$a1parf, input$a2parf, input$a3parf, input$a4parf,
                                  input$bpar1f, input$bpar2f, input$bpar3f, input$bpar4f)
                     } else {
                         stop('Classical parameterization not available for selected item class')
@@ -254,6 +264,7 @@ shinyItemplot <- function(){
                     par[names(par) == 'a1'] <- input$a1par
                     par[names(par) == 'a2'] <- input$a2par
                     par[names(par) == 'd'] <- input$dpar
+                    par[names(par) == 'd'] <- input$idpar
                     par[names(par) == 'g'] <- logit(input$gpar)
                     par[names(par) == 'u'] <- logit(input$upar)
                     par[names(par) == 'd0'] <- input$d0par
@@ -268,14 +279,14 @@ shinyItemplot <- function(){
                 mod@pars[[1]]@par <- par
                 mod
                 }
-            
-            output$main_plot <- renderPlot({
+
+            output$main_plot <- shiny::renderPlot({
                 mod <- genmod(input)
                 print(itemplot(mod, 1, type=input$plottype, rotate = 'none',
                                theta_lim=c(input$theta_lim_low, input$theta_lim_high)))
             })
-            
-            output$coefs <- renderPrint({
+
+            output$coefs <- shiny::renderPrint({
                 mod <- genmod(input)
                 cat('Item parameters: \n\n')
                 print(coef(mod, rotate = 'none')[[1L]])
@@ -284,9 +295,9 @@ shinyItemplot <- function(){
                     print(coef(mod, IRTpars = TRUE)[[1L]])
                 }
             })
-            
+
         }
-                )
-    
+    )
+
     return(ret)
 }
