@@ -44,7 +44,7 @@ draw.thetas <- function(theta0, pars, fulldata, itemloc, cand.t.var, prior.t.var
                                    offterm=OffTerm, CUSTOM.IND=CUSTOM.IND)
     itemtrace1 <- computeItemtrace(pars=pars, Theta=theta1, itemloc=itemloc,
                                    offterm=OffTerm, CUSTOM.IND=CUSTOM.IND)
-    totals <- .Call('denRowSums', fulldata, itemtrace0, itemtrace1, log_den0, 
+    totals <- .Call('denRowSums', fulldata, itemtrace0, itemtrace1, log_den0,
                     log_den1, mirtClusterEnv$ncores)
     total_0 <- totals[[1L]]
     total_1 <- totals[[2L]]
@@ -65,7 +65,7 @@ draw.thetas <- function(theta0, pars, fulldata, itemloc, cand.t.var, prior.t.var
 imputePars <- function(pars, covB, imputenums, constrain){
     shift <- mirt_rmvnorm(1L, sigma=covB)
     for(i in 1L:length(pars)){
-        pn <- pars[[i]]@parnum 
+        pn <- pars[[i]]@parnum
         pick2 <- imputenums %in% pn
         pick1 <- pn %in% imputenums
         pars[[i]]@par[pick1] <- pars[[i]]@par[pick1] + shift[pick2]
@@ -203,18 +203,13 @@ d2r <-function(d) pi*d/180
 closeEnough <- function(x, low, up) all(x >= low & x <= up)
 
 logit <- function(x){
-    ret <- log(x / (1 - x))
+    ret <- qlogis(x)
     ret <- ifelse(x == 0, -999, ret)
     ret <- ifelse(x == 1, 999, ret)
     ret
 }
 
-antilogit <- function(x){
-    ret <- 1 / (1 + exp(-x))
-    ret <- ifelse(x == -999, 0, ret)
-    ret <- ifelse(x == 999, 1, ret)
-    ret
-}
+antilogit <- function(x) plogis(x)
 
 test_info <- function(pars, Theta, Alist, K){
     infolist <- list()
@@ -236,24 +231,31 @@ test_info <- function(pars, Theta, Alist, K){
 
 Lambdas <- function(pars, Names, explor = FALSE, alpha = .05){
     J <- length(pars) - 1L
-    lambdas <- lowerlambdas <- upperlambdas <- 
+    lambdas <- lowerlambdas <- upperlambdas <-
         matrix(NA, J, length(ExtractLambdas(pars[[1L]])))
+    gcov <- ExtractGroupPars(pars[[J+1L]])$gcov
+    if(ncol(gcov) < ncol(lambdas)){
+        tmpcov <- diag(ncol(lambdas))
+        tmpcov[1L:ncol(gcov), 1L:ncol(gcov)] <- gcov
+        gcov <- tmpcov
+    }
+    chl <- chol(gcov)
     z <- qnorm(1 - alpha/2)
     rownames(lambdas) <- rownames(upperlambdas) <- rownames(lowerlambdas) <- Names
     for(i in 1L:J){
         tmp <- pars[[i]]
-        lambdas[i,] <- ExtractLambdas(tmp)/1.702
+        lambdas[i,] <- ExtractLambdas(tmp) /1.702
         tmp@par <- pars[[i]]@par - z * pars[[i]]@SEpar
-        lowerlambdas[i,] <- ExtractLambdas(tmp)/1.702
+        lowerlambdas[i,] <- ExtractLambdas(tmp) /1.702
         tmp@par <- pars[[i]]@par + z * pars[[i]]@SEpar
-        upperlambdas[i,] <- ExtractLambdas(tmp)/1.702
+        upperlambdas[i,] <- ExtractLambdas(tmp) /1.702
     }
-    norm <- sqrt(1 + rowSums(lambdas^2))
+    norm <- sqrt(1 + rowSums((lambdas%*%chl)^2))
     F <- as.matrix(lambdas/norm)
     if(!explor){
         norml <- sqrt(1 + rowSums(lowerlambdas^2, na.rm=TRUE))
         normh <- sqrt(1 + rowSums(upperlambdas^2, na.rm=TRUE))
-        ret <- list(F=F, lower=as.matrix(lowerlambdas/norml), 
+        ret <- list(F=F, lower=as.matrix(lowerlambdas/norml),
                     upper=as.matrix(upperlambdas/normh))
     } else {
         ret <- list(F=F, lower=list(), upper=list())
@@ -308,7 +310,7 @@ bfactor2mod <- function(model, J){
     return(model)
 }
 
-updatePrior <- function(pars, Theta, Thetabetween, list, ngroups, nfact, J, 
+updatePrior <- function(pars, Theta, Thetabetween, list, ngroups, nfact, J,
                         BFACTOR, sitems, cycles, rlist, prior){
     Prior <- Priorbetween <- vector('list', ngroups)
     if(list$EH){
@@ -647,7 +649,7 @@ ReturnPars <- function(PrepList, itemnames, random, MG = FALSE){
             est <- c(est, tmpgroup[[i]]@est)
             lbound <- c(lbound, tmpgroup[[i]]@lbound)
             ubound <- c(ubound, tmpgroup[[i]]@ubound)
-            tmp <- sapply(as.character(tmpgroup[[i]]@prior.type), 
+            tmp <- sapply(as.character(tmpgroup[[i]]@prior.type),
                                  function(x) switch(x, '1'='norm', '2'='lnorm', '3'='beta', 'none'))
             prior.type <- c(prior.type, tmp)
             prior_1 <- c(prior_1, tmpgroup[[i]]@prior_1)
@@ -663,7 +665,7 @@ ReturnPars <- function(PrepList, itemnames, random, MG = FALSE){
             est <- c(est, random[[i]]@est)
             lbound <- c(lbound, random[[i]]@lbound)
             ubound <- c(ubound, random[[i]]@ubound)
-            tmp <- sapply(as.character(random[[i]]@prior.type), 
+            tmp <- sapply(as.character(random[[i]]@prior.type),
                           function(x) switch(x, '1'='norm', '2'='lnorm', '3'='beta', 'none'))
             prior.type <- c(prior.type, tmp)
             prior_1 <- c(prior_1, random[[i]]@prior_1)
@@ -682,6 +684,9 @@ ReturnPars <- function(PrepList, itemnames, random, MG = FALSE){
 
 UpdatePrepList <- function(PrepList, pars, random, MG = FALSE){
     currentDesign <- ReturnPars(PrepList, PrepList[[1L]]$itemnames, random=random, MG = TRUE)
+    if(nrow(currentDesign) != nrow(pars))
+        stop('Rows in supplied and starting value data.frame objects do not match. Were the
+             data or itemtype input arguments modified?')
     if(!all(as.matrix(currentDesign[,c('group', 'item', 'class', 'name', 'parnum')]) ==
                 as.matrix(pars[,c('group', 'item', 'class', 'name', 'parnum')])))
         stop('Critical internal parameter labels do not match those returned from pars = \'values\'')
@@ -703,8 +708,8 @@ UpdatePrepList <- function(PrepList, pars, random, MG = FALSE){
                 PrepList[[g]]$pars[[i]]@est[j] <- as.logical(pars[ind,'est'])
                 PrepList[[g]]$pars[[i]]@lbound[j] <- pars[ind,'lbound']
                 PrepList[[g]]$pars[[i]]@ubound[j] <- pars[ind,'ubound']
-                tmp <- as.character(pars[ind,'prior.type'])                
-                PrepList[[g]]$pars[[i]]@prior.type[j] <- 
+                tmp <- as.character(pars[ind,'prior.type'])
+                PrepList[[g]]$pars[[i]]@prior.type[j] <-
                     switch(tmp, norm=1L, lnorm=2L, beta=3L, 0L)
                 PrepList[[g]]$pars[[i]]@prior_1[j] <- pars[ind,'prior_1']
                 PrepList[[g]]$pars[[i]]@prior_2[j] <- pars[ind,'prior_2']
@@ -879,7 +884,7 @@ maketabData <- function(stringfulldata, stringtabdata, group, groupNames, nitem,
         if(!is.null(survey.weights)){
             mtc <- match(tmpstringdata, stringtabdata)
             Freq <- mySapply(1L:nrow(tabdata), function(x, std, tstd, w)
-                sum(w[stringtabdata[x] == tstd]), std=stringtabdata, tstd=tmpstringdata, 
+                sum(w[stringtabdata[x] == tstd]), std=stringtabdata, tstd=tmpstringdata,
                 w=survey.weights)
         } else {
             Freq[stringtabdata %in% tmpstringdata] <- as.integer(table(
@@ -922,11 +927,13 @@ makeopts <- function(method = 'MHRM', draws = 2000L, calcLL = TRUE, quadpts = NU
                      SEtol = .001, grsm.block = NULL, D = 1, TOL = NULL,
                      rsm.block = NULL, calcNull = TRUE, BFACTOR = FALSE,
                      technical = list(), use = 'pairwise.complete.obs',
-                     SE.type = 'crossprod', large = NULL, accelerate = TRUE, empiricalhist = FALSE,
-                     optimizer = NULL, ...)
+                     SE.type = 'crossprod', large = NULL, accelerate = 'Ramsay', empiricalhist = FALSE,
+                     optimizer = NULL, solnp_args = list(), alabama_args = list(), ...)
 {
     opts <- list()
     if(method == 'MHRM' || method == 'MIXED') SE.type <- 'MHRM'
+    if(!(method %in% c('MHRM', 'MIXED', 'BL', 'EM', 'QMCEM')))
+        stop('method argument not supported')
     D <- 1
     opts$method = method
     opts$draws = draws
@@ -945,10 +952,10 @@ makeopts <- function(method = 'MHRM', draws = 2000L, calcLL = TRUE, quadpts = NU
     opts$customPriorFun = technical$customPriorFun
     opts$BFACTOR = BFACTOR
     opts$accelerate = accelerate
-    opts$TOL <- ifelse(is.null(TOL), if(method == 'EM') 1e-4 else 
+    opts$TOL <- ifelse(is.null(TOL), if(method == 'EM' || method == 'QMCEM') 1e-4 else
         if(method == 'BL') 1e-8 else 1e-3, TOL)
     if(SE.type == 'SEM' && SE){
-        opts$accelerate <- FALSE
+        opts$accelerate <- 'none'
         if(is.null(TOL)) opts$TOL <- 1e-5
         if(is.null(technical$NCYCLES)) technical$NCYCLES <- 1000L
     }
@@ -960,7 +967,7 @@ makeopts <- function(method = 'MHRM', draws = 2000L, calcLL = TRUE, quadpts = NU
     opts$technical$parallel <- ifelse(is.null(technical$parallel), TRUE, technical$parallel)
     opts$MAXQUAD <- ifelse(is.null(technical$MAXQUAD), 10000L, technical$MAXQUAD)
     opts$NCYCLES <- ifelse(is.null(technical$NCYCLES), 2000L, technical$NCYCLES)
-    if(opts$method == 'EM')
+    if(opts$method %in% c('EM', 'QMCEM'))
         opts$NCYCLES <- ifelse(is.null(technical$NCYCLES), 500L, technical$NCYCLES)
     opts$BURNIN <- ifelse(is.null(technical$BURNIN), 150L, technical$BURNIN)
     opts$SEMCYCLES <- ifelse(is.null(technical$SEMCYCLES), 50, technical$SEMCYCLES)
@@ -973,6 +980,7 @@ makeopts <- function(method = 'MHRM', draws = 2000L, calcLL = TRUE, quadpts = NU
         if(is.null(opts$quadpts)) opts$quadpts <- 199L
         if(opts$NCYCLES == 500L) opts$NCYCLES <- 2000L
     }
+    if(method == 'QMCEM' && is.null(opts$quadpts)) opts$quadpts <- 2000L
     opts$MSTEPTOL <- ifelse(is.null(technical$MSTEPTOL), opts$TOL/1000, technical$MSTEPTOL)
     if(opts$method == 'MHRM' || opts$method =='MIXED' || SE.type == 'MHRM')
         set.seed(12345L)
@@ -987,10 +995,22 @@ makeopts <- function(method = 'MHRM', draws = 2000L, calcLL = TRUE, quadpts = NU
     opts$returnPrepList <- FALSE
     opts$PrepList <- NULL
     if(is.null(optimizer)){
-        opts$Moptim <- if(method == 'EM' || method == 'BL') 'BFGS' else 'NR'
+        opts$Moptim <- if(method %in% c('EM','BL','QMCEM')) 'BFGS' else 'NR'
     } else {
         opts$Moptim <- optimizer
     }
+    if(opts$Moptim == 'solnp'){
+        if(is.null(solnp_args$control)) solnp_args$control <- list()
+        if(is.null(solnp_args$control$trace)) solnp_args$control$trace <- 0
+        if(method != 'EM') stop('solnp only supported for optimization with EM algorithm')
+        opts$solnp_args <- solnp_args
+    } else if(opts$Moptim == 'alabama'){
+        if(method != 'EM') stop('alabama only supported for optimization with EM algorithm')
+        if(is.null(alabama_args$control.outer)) alabama_args$control.outer <- list()
+        if(is.null(alabama_args$control.optim)) alabama_args$control.optim <- list()
+        if(is.null(alabama_args$control.outer$trace)) alabama_args$control.outer$trace <- FALSE
+        opts$solnp_args <- alabama_args
+    }
     if(!is.null(large)){
         if(is.logical(large))
             if(large) opts$returnPrepList <- TRUE
@@ -1000,7 +1020,7 @@ makeopts <- function(method = 'MHRM', draws = 2000L, calcLL = TRUE, quadpts = NU
     return(opts)
 }
 
-reloadPars <- function(longpars, pars, ngroups, J){    
+reloadPars <- function(longpars, pars, ngroups, J){
     if(FALSE){
         pars <- .Call('reloadPars', longpars, pars, ngroups, J)
     } else {
@@ -1177,10 +1197,10 @@ smooth.cor <- function(x){
 }
 
 RMSEA.CI <- function(X2, df, N, ci.lower=.05, ci.upper=.95) {
-    
+
     lower.lambda <- function(lambda) pchisq(X2, df=df, ncp=lambda) - ci.upper
     upper.lambda <- function(lambda) pchisq(X2, df=df, ncp=lambda) - ci.lower
-    
+
     lambda.l <- try(uniroot(f=lower.lambda, lower=0, upper=X2)$root, silent=TRUE)
     lambda.u <- try(uniroot(f=upper.lambda, lower=0, upper=max(N, X2*5))$root, silent=TRUE)
     if(!is(lambda.l, 'try-error')){
@@ -1193,7 +1213,7 @@ RMSEA.CI <- function(X2, df, N, ci.lower=.05, ci.upper=.95) {
     } else {
         RMSEA.upper <- 0
     }
-    
+
     return(c(RMSEA.lower, RMSEA.upper))
 }
 
@@ -1202,7 +1222,7 @@ assignInformationMG <- function(object){
     names <- colnames(object@information)
     spl_names <- strsplit(names, split="\\.")
     spl_names_par <- sapply(spl_names, function(x) x[1L])
-    spl_names <- lapply(spl_names, 
+    spl_names <- lapply(spl_names,
                         function(x) as.numeric(x[-1L]))
     spl_names <- do.call(rbind, spl_names)
     for(g in 1L:length(object@pars)){
@@ -1211,7 +1231,7 @@ assignInformationMG <- function(object){
             object@pars[[g]]@pars[[J+1L]]@parnum)]
         pick <- spl_names[,g] >= from & spl_names[,g] <= to
         tmp <- object@information[pick,pick]
-        colnames(tmp) <- rownames(tmp) <- 
+        colnames(tmp) <- rownames(tmp) <-
             paste(spl_names_par[pick], spl_names[pick,g], sep='.')
         object@pars[[g]]@information <- tmp
     }
@@ -1242,13 +1262,13 @@ BL.LL <- function(p, est, longpars, pars, ngroups, J, Theta, PrepList, specific,
     LL <- 0
     for(g in 1L:ngroups){
         if(BFACTOR){
-            expected <- Estep.bfactor(pars=pars2[[g]], 
+            expected <- Estep.bfactor(pars=pars2[[g]],
                                       tabdata=Data$tabdatalong, freq=Data$Freq[[g]],
                                       Theta=Theta, prior=prior[[g]],
                                       specific=specific, sitems=sitems,
                                       itemloc=itemloc, CUSTOM.IND=CUSTOM.IND)$expected
         } else {
-            expected <- Estep.mirt(pars=pars2[[g]], 
+            expected <- Estep.mirt(pars=pars2[[g]],
                                    tabdata=Data$tabdatalong, freq=Data$Freq[[g]],
                                    Theta=Theta, prior=Prior[[g]], itemloc=itemloc,
                                    CUSTOM.IND=CUSTOM.IND)$expected
@@ -1258,14 +1278,20 @@ BL.LL <- function(p, est, longpars, pars, ngroups, J, Theta, PrepList, specific,
     LL
 }
 
+select_quadpts <- function(nfact) switch(as.character(nfact),
+                                         '1'=61, '2'=31, '3'=15, '4'=9, '5'=7, 3)
+
+select_quadpts2 <- function(nfact) switch(as.character(nfact),
+                                          '1'=41, '2'=21, '3'=11, '4'=7, '5'=5, 3)
+
 mirt_rmvnorm <- function(n, mean = rep(0, nrow(sigma)), sigma = diag(length(mean)),
                          check = FALSE)
-{    
-    # Version modified from mvtnorm::rmvnorm, version 0.9-9996, 19-April, 2014. 
+{
+    # Version modified from mvtnorm::rmvnorm, version 0.9-9996, 19-April, 2014.
     if(check){
-        if (!isSymmetric(sigma, tol = sqrt(.Machine$double.eps), check.attributes = FALSE)) 
+        if (!isSymmetric(sigma, tol = sqrt(.Machine$double.eps), check.attributes = FALSE))
             stop("sigma must be a symmetric matrix")
-        if (length(mean) != nrow(sigma)) 
+        if (length(mean) != nrow(sigma))
             stop("mean and sigma have non-conforming size")
     }
     ev <- eigen(sigma, symmetric = TRUE)
@@ -1281,7 +1307,7 @@ mirt_rmvnorm <- function(n, mean = rep(0, nrow(sigma)), sigma = diag(length(mean
 
 mirt_dmvnorm <- function(x, mean, sigma, log = FALSE)
 {
-    # Version modified from mvtnorm::dmvnorm, version 0.9-9996, 19-April, 2014. 
+    # Version modified from mvtnorm::dmvnorm, version 0.9-9996, 19-April, 2014.
     if(is.vector(x)) x <- matrix(x, nrow=1L)
     if (missing(mean)) mean <- rep(0, length = ncol(x))
     if (missing(sigma)) sigma <- diag(ncol(x))
@@ -1293,6 +1319,148 @@ mirt_dmvnorm <- function(x, mean, sigma, log = FALSE)
     exp(logretval)
 }
 
+# prior for latent class analysis
+lca_prior <- function(Theta, Etable){
+    TP <- nrow(Theta)
+    if ( is.null(Etable) ){
+        prior <- rep( 1/TP , TP )
+    } else {
+        prior <- rowSums(Etable)
+    }
+    prior <- prior / sum(prior)
+    return(prior)
+}
+
+makeObstables <- function(dat, K){
+    ret <- vector('list', ncol(dat))
+    sumscore <- rowSums(dat)
+    for(i in 1L:length(ret)){
+        ret[[i]] <- matrix(0, sum(K-1L)+1L, K[i])
+        colnames(ret[[i]]) <- paste0(1L:K[i]-1L)
+        rownames(ret[[i]]) <- paste0(1L:nrow(ret[[i]])-1L)
+        split <- by(sumscore, dat[,i], table)
+        for(j in 1L:K[i]){
+            m <- match(names(split[[j]]), rownames(ret[[i]]))
+            ret[[i]][m,j] <- split[[j]]
+        }
+        ret[[i]] <- ret[[i]][-c(1L, nrow(ret[[i]])), ]
+    }
+    ret
+}
+
+collapseCells <- function(O, E, mincell = 1){
+    for(i in 1L:length(O)){
+        On <- O[[i]]
+        En <- E[[i]]
+        drop <- which(rowSums(is.na(En)) > 0)
+        En[is.na(En)] <- 0
+
+        #collapse known upper and lower sparce cells
+        if(length(drop) > 0L){
+            up <- drop[1L]:drop[length(drop)/2]
+            low <- drop[length(drop)/2 + 1L]:drop[length(drop)]
+            En[max(up)+1, ] <- colSums(En[c(up, max(up)+1), , drop = FALSE])
+            On[max(up)+1, ] <- colSums(On[c(up, max(up)+1), , drop = FALSE])
+            En[min(low)-1, ] <- colSums(En[c(low, min(low)-1), , drop = FALSE])
+            On[min(low)-1, ] <- colSums(On[c(low, min(low)-1), , drop = FALSE])
+            En[c(up, low), ] <- On[c(up, low), ] <- NA
+            En <- na.omit(En)
+            On <- na.omit(On)
+        }
+
+        #drop 0's and 1's
+        drop <- rowSums(On) == 0L
+        On <- On[!drop,]
+        En <- En[!drop,]
+        L <- En < mincell
+        drop <- c()
+        for(j in 1L:(nrow(On)-1L)){
+            ss <- sum(On[j,])
+            if(ss == 1L){
+                drop <- c(drop, j)
+                On[j+1L, ] <- On[j+1L, ] + On[j, ]
+                En[j+1L, ] <- En[j+1L, ] + En[j, ]
+            }
+        }
+        if(length(drop)){
+            On <- On[-drop,]
+            En <- En[-drop,]
+        }
+        ss <- sum(On[nrow(On),])
+        if(ss == 1L){
+            On[nrow(On)-1L, ] <- On[nrow(On)-1L, ] + On[nrow(On), ]
+            En[nrow(On)-1L, ] <- En[nrow(On)-1L, ] + En[nrow(On), ]
+            On <- On[-nrow(On),]; En <- En[-nrow(En),]
+        }
+
+        #collapse accross as much as possible
+        if(ncol(En) > 2L){
+            for(j in 1L:nrow(En)){
+                if(!any(L[j,])) next
+                tmp <- En[j, ]
+                tmp2 <- On[j, ]
+                while(length(tmp) > 2L){
+                    m <- min(tmp)
+                    whc <- which(m == tmp)
+                    if(whc == 1L){
+                        tmp[2L] <- tmp[2L] + tmp[1L]
+                        tmp2[2L] <- tmp2[2L] + tmp2[1L]
+                    } else if(whc == length(tmp)){
+                        tmp[length(tmp)-1L] <- tmp[length(tmp)-1L] + tmp[length(tmp)]
+                        tmp2[length(tmp2)-1L] <- tmp2[length(tmp2)-1L] + tmp2[length(tmp2)]
+                    } else {
+                        left <- min(tmp[whc-1L], tmp[whc+1L]) == c(tmp[whc-1L], tmp[whc+1L])[1L]
+                        pick <- if(left) whc-1L else whc+1L
+                        tmp[pick] <- tmp[pick] + tmp[whc]
+                        tmp2[pick] <- tmp2[pick] + tmp2[whc]
+                    }
+                    tmp[whc] <- tmp2[whc] <- NA
+                    tmp <- na.omit(tmp); tmp2 <- na.omit(tmp2)
+                    if(all(tmp >= mincell)) break
+                }
+                tmp <- c(tmp, rep(NA, ncol(En)-length(tmp)))
+                tmp2 <- c(tmp2, numeric(ncol(En)-length(tmp2)))
+                En[j, ] <- tmp
+                On[j, ] <- tmp2
+            }
+        }
+
+        #merge across
+        En[is.na(En)] <- 0
+        L <- En < mincell & En != 0
+        while(any(L, na.rm = TRUE)){
+            whc <- min(which(rowSums(L) > 0L))
+            if(whc == 1L){
+                En[2L,] <- En[2L, ] + En[1L,]
+                On[2L,] <- On[2L, ] + On[1L,]
+                En <- En[-1L,]; On <- On[-1L,]
+            } else if(whc == nrow(En)){
+                En[nrow(En)-1L,] <- En[nrow(En)-1L, ] + En[nrow(En),]
+                On[nrow(En)-1L,] <- On[nrow(En)-1L, ] + On[nrow(En),]
+                En <- En[-nrow(En),]; On <- On[-nrow(En),]
+            } else {
+                ss <- c(sum(On[whc-1L,]), sum(On[whc+1L,]))
+                up <- (min(ss) == ss)[1L]
+                pick <- if(up) whc-1L else whc+1L
+                En[pick,] <- En[pick, ] + En[whc,]
+                On[pick,] <- On[pick, ] + On[whc,]
+                En <- En[-whc,]; On <- On[-whc,]
+            }
+            L <- En < mincell & En != 0
+        }
+        En[En == 0] <- NA
+        E[[i]] <- En
+        O[[i]] <- On
+    }
+    return(list(O=O, E=E))
+}
+
+rmsea <- function(X2, df, N){
+    ret <- ifelse((X2 - df) > 0,
+                  sqrt(X2 - df) / sqrt(df * (N-1)), 0)
+    ret
+}
+
 mirtClusterEnv <- new.env()
 mirtClusterEnv$ncores <- 1L
 
